# -*- coding: utf-8 -*-

from PyQt4 import QtCore, QtGui
import codecs
import string
import sys
import webbrowser
import os
import cPickle

class TamilConverter ():
	chars = {	u'`a' : u'அ', u'`A' : u'ஆ', u'`i' : u'இ', u'`I' : u'ஈ', u'`u' : u'உ', u'`U' : u'ஊ', 
			u'`e' : u'எ', u'`E' : u'ஏ', u'^I' : u'ஐ', u'`o' : u'ஒ', u'`O' : u'ஓ', u'^O' : u'ஔ',
			u'k' : u'க', u'c' : u'ச', u'd' : u'ட', u'T' : u'ட', u't' : u'த', u'b' : u'ப', u'p' : u'ப', 
			u'r' : u'ர', u'g' : u'ங', u'j' : u'ஞ', u'N' : u'ண', u'`n' : u'ந', u'm' : u'ம', u'n' : u'ன', 
			u'y' : u'ய', u'R' : u'ற', u'l' : u'ல', u'v' : u'வ', u'z' : u'ழ', u'L' : u'ள', u'S' : u'ஷ', 
			u's' : u'ஸ',
			u'h' : u'ஹ',
			u'J' : u'ஜ',
			u'a' : u'ா', 
			u'i' : u'ி', 
			u'I' : u'ீ',
			u'u' : u'ு',
			u'U' : u'ூ',
			u'e' : u'ெ',
			u'E' : u'ே',
			u'A' : u'ை',
			u'o' : u'ொ',
			u'O' : u'ோ',
			u'w' : u'ௌ',
			u'~' : u'்',
			u'x' : u'ஃ'
		 }
		 
	def doConvert (self, convText):
		unitxt = list (convText)
		result = list ()
		prevchar = ''
		for char in unitxt:
			if char == u'`' or char == u'^':
				prevchar = char
				continue
			
			if prevchar == u'`' or prevchar == u'^':
				key = prevchar + char
				if self.chars.has_key (key):
					result.append (self.chars[key])
			else:
				if self.chars.has_key (char):
					result.append (self.chars[char])
				else:
					result.append (char)
			prevchar = char
		return string.join (result, '')


class MainWindow (QtGui.QMainWindow):
	modified_flag = False
	current_file = None
	
	def showEvent (self, event):
		"""Event for form showing - update the font from config file """
		self.printer = QtGui.QPrinter ()
		if os.path.exists (os.path.expanduser ("~/.PyTamEditor")):
			fp = file (os.path.expanduser ("~/.PyTamEditor"), "rb")
			font = QtGui.QFont ()
			fdesc = cPickle.load (fp)
			font.fromString (fdesc)
			txtOutput = self.findChild (QtGui.QTextEdit, "txtTamil")
			txtOutput.setFont (font)
			fp.close ()
			
	def onEditUndo (self):
		txtInput = self.findChild (QtGui.QPlainTextEdit, "txtEnglish")
		txtInput.undo ()
	
	def onEditRedo (self):
		txtInput = self.findChild (QtGui.QPlainTextEdit, "txtEnglish")
		txtInput.redo ()
	
	def onEditCut (self):
		txtInput = self.findChild (QtGui.QPlainTextEdit, "txtEnglish")
		txtInput.cut ()
	
	def onEditCopy (self):
		txtInput = self.findChild (QtGui.QPlainTextEdit, "txtEnglish")
		txtInput.copy ()
	
	def onEditPaste (self):
		txtInput = self.findChild (QtGui.QPlainTextEdit, "txtEnglish")
		txtInput.paste ()
		
	def onEditCopyTamil (self):
		txtOutput = self.findChild (QtGui.QTextEdit, "txtTamil")
		txtOutput.selectAll ()
		txtOutput.copy ()
	
	def onFormatFont (self):
		"""Font choosing action event"""
		txtOutput = self.findChild (QtGui.QTextEdit, "txtTamil")
		font, ok = QtGui.QFontDialog.getFont (txtOutput.font(), self, "Choose Tamil (Unicode) Font")
		txtOutput.setFont (font)
		fp = file (os.path.expanduser ("~/.PyTamEditor"), "wb")
		cPickle.dump (font.toString (), fp, -1)
		fp.close ()

	def onFileNew (self):
		"""File New action event"""
		txtInput = self.findChild (QtGui.QPlainTextEdit, "txtEnglish")
		txtOutput = self.findChild (QtGui.QTextEdit, "txtTamil")
		if self.modified_flag == True:
			res = QtGui.QMessageBox.question (self, "File changed", "File has changed. Are you sure you wish to lose the current changes?", "OK", "Cancel")
			if res == 1:
				return
		
		txtInput.setPlainText ("")
		txtOutput.setPlainText ("")
		self.setWindowTitle ("PyTamEditor - Untitled")
		self.modified_flag = False
		self.current_file = None
			
	def saveFile (self, filename):
		"""Procedure to save a file"""
		txtInput = self.findChild (QtGui.QPlainTextEdit, "txtEnglish")
		fcontents = txtInput.toPlainText ()
		f = file (filename, "w")
		f.write (fcontents)
		f.close ()
		self.modified_flag = False
		self.current_file = filename
		
	def exportFile (self, filename):
		"""Procedure to export the UNICODE contents to a file"""
		txtOutput = self.findChild (QtGui.QTextEdit, "txtTamil")
		
		#fcontents = unicode (txtOutput.toPlainText().toUtf8(), "utf-8")		
		#f = codecs.open (filename, "w", encoding="utf-8")
		
		f = file (filename, "w")
		f.write (txtOutput.toPlainText().toUtf8())
		
		#f.write ( fcontents )
		f.close ()
		
	def openFile (self, filename):
		"""Procedure to open a file"""
		txtInput = self.findChild (QtGui.QPlainTextEdit, "txtEnglish")
		f = file (filename, "r")
		fcontents = f.read ()
		f.close ()
		txtInput.setPlainText (fcontents)
		self.modified_flag = False
		self.current_file = filename
			
	def onFileSaveAs (self):
		"""Save As action event"""
		filename = QtGui.QFileDialog.getSaveFileName (self, "Save As", os.path.expanduser("~"))
		if filename <> "":
			self.saveFile (filename)
			self.setWindowTitle ("PyTamEditor - " + filename)
			
	def onFileSave (self):
		"""Save action event"""
		if self.modified_flag == True:
			if self.current_file == None:
				self.onFileSaveAs ()
			else:
				self.saveFile (self.current_file)
				
	def onFileOpen (self):
		"""Open action event"""
		if self.modified_flag == True:
			res = QtGui.QMessageBox.question (self, "File changed", "File has changed. Are you sure you wish to open another file without saving?", "OK", "Cancel")
			if res == 1:
				return
		
		filename = QtGui.QFileDialog.getOpenFileName (self, "Open", os.path.expanduser("~"))
		if filename <> "":
			self.openFile (filename)
			self.setWindowTitle ("PyTamEditor - " + filename)
			
	def onFileExport (self):
		"""Export as Unicode action event"""
		filename = QtGui.QFileDialog.getSaveFileName (self, "Export")
		if filename <> "":
			self.exportFile (filename)
			
	def onFilePrint (self):
		"""File printing action event"""
		dlg = QtGui.QPrintDialog (self.printer, self)
		if dlg.exec_ ():
			txtOutput = self.findChild (QtGui.QTextEdit, "txtTamil")
			txtOutput.print_ (self.printer)

	def onFilePageSetup (self):
		dlg = QtGui.QPageSetupDialog (self.printer, self)
		dlg.exec_ ()

	def closeEvent (self, event):
		if self.modified_flag == True:
			res = QtGui.QMessageBox.question (self, "File changed", "File has changed. Are you sure you wish to exit without saving?", "OK", "Cancel")
			if res == 0:
				event.accept ()
			else:
				event.ignore ()
		else:
			event.accept ()		
		
	def onFileExit (self):
		"""File exit action event"""
		self.close ()
		
	def onHelpAbout (self):
		"""Help about action event"""
		QtGui.QMessageBox.about (self, "About PyTamEditor", "A simple Tamil Unicode Editor with English phonetic key mappings, written in Python/PyQt4.\n\n Copyright (c) V.Harishankar (http://harishankar.org).\nLicensed under the GNU/GPL v3")
		
	def onHelpReadme (self):
		if os.path.exists ("readme.html"):
			ret = webbrowser.open ("readme.html")
		else:
			QtGui.QMessageBox.warning (self, "Cannot Find Readme", "The file readme.html is missing or you are running this script outside its working directory.")
	
	def onTextChanged (self):
		"""Event for text changing"""
		self.modified_flag = True
		
		txtInput = self.findChild (QtGui.QPlainTextEdit, "txtEnglish")
		txtOutput = self.findChild (QtGui.QTextEdit, "txtTamil")
		str_input = unicode (txtInput.toPlainText (), "utf-8")
		tc = TamilConverter ()
		str_output = tc.doConvert (str_input)
		txtOutput.setPlainText (str_output)
		